#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <ogcsys.h>
#include <gccore.h>
#include <unistd.h>

#include "dvd.h"
#include "loadapp.h"
#include "gameloader.h"
#include "loadmios.h"
#include "background_rnc.h"
#include "screen.h"
#include "loader.h"
#include "debugprintf.h"
#include "dvd.h"

#include "dvdreadpatch.h"
#include "systemconfig.h"
#include "video.h"
#include "Toy.h"

#define BACKGROUND_SIZE 563200
#define HOMEBREW_MAGIC 0x807FFFE0

static void *xfb = NULL;
static GXRModeObj *rmode = NULL;
static u8 *gfxbuffer;

static dvdinfo_t gcm_disk __attribute((aligned(32)));

void clearScreen(void)
{
	int i;

	console_init(xfb, 20, 20, rmode->fbWidth, rmode->xfbHeight,
		rmode->fbWidth * VI_DISPLAY_PIX_SZ);
	if (gfxbuffer != NULL) {
		VIDEO_ClearFrameBuffer(rmode, xfb,
			*((unsigned long *) &gfxbuffer[BACKGROUND_SIZE - 4]));
		memcpy(xfb, gfxbuffer, BACKGROUND_SIZE);
		for (i = 0; i < 17; i++) {
			printf("\n");
		}
		printf("\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t\t%s", GC_VERSION);
		for (i = 0; i < 3; i++) {
			printf("\n");
		}
	} else {
		VIDEO_ClearFrameBuffer(rmode, xfb, COLOR_BLACK);
	}
}

void check_for_homebrew(void)
{
	char *magic = (char *) HOMEBREW_MAGIC;

	if (strncmp(magic, "gchomebrew dol", 32) == 0) {
		/* Start homebrew. */
		load_dol();
	}
}

void start_from_NeoGamma(void)
{
	base_offset = 0;
	int rv;
	
	//rv = dvd_read(checkBuffer, 32, 0x8001);
	//DVD_CHECK();

	base_offset = *(u32 *)0x807FFF20;
	second_disc_offset = *(u32 *)0x807FFF24;
	set_patchmode(*(u32 *)0x807FFF30);

	rv = dvd_read(&gcm_disk, sizeof(gcm_disk), 0);
	DVD_CHECK();
	
	setup_video(*(u32 *)0x807FFF28);
	load_app(0);
}

int main(int argc, char **argv)
{
	VIDEO_Init();
	PAD_Init();

	if (strcmp((char *)0x807FFF00, "GAMECUBELOADER01") == 0)
	{
		switch(*(u32 *)0x807FFF28)
		{
			case VIDEO_MODE_PAL60:
				rmode = &TVEurgb60Hz480IntDf;
				*(u32*)0x800000CC = 5;
			break;
			
			case VIDEO_MODE_PAL:
				rmode = &TVPal528IntDf;
				*(u32*)0x800000CC = 1;
			break;
			
			case VIDEO_MODE_NTSC480P:
				rmode = &TVNtsc480Prog;
				*(u32*)0x800000CC = 0;
			break;
			
			case VIDEO_MODE_PAL480P:
				rmode = &TVEurgb60Hz480Prog;
				*(u32*)0x800000CC = 5;
			break;

			case VIDEO_MODE_NTSC:
			default:
				rmode = &TVNtsc480IntDf;
				*(u32*)0x800000CC = 0;
			break;
		}
	} else
	{
		rmode = VIDEO_GetPreferredMode(NULL);
	}

	xfb = MEM_K0_TO_K1(SYS_AllocateFramebuffer(rmode));
	console_init(xfb, 20, 20, rmode->fbWidth, rmode->xfbHeight,
		rmode->fbWidth * VI_DISPLAY_PIX_SZ);

	VIDEO_Configure(rmode);
	VIDEO_SetNextFramebuffer(xfb);
	VIDEO_SetBlack(FALSE);
	VIDEO_Flush();
	VIDEO_WaitVSync();
	if (rmode->viTVMode & VI_NON_INTERLACE)
		VIDEO_WaitVSync();

	/* Setup Graphics */
	gfxbuffer = (u8 *) memalign(32, BACKGROUND_SIZE);
	if (gfxbuffer == NULL) {
		printf("Failed to allocated graphics memory\n");
	} else {
		depackrnc1((u8 *) background_rnc, gfxbuffer);
	}
	debug_printf("gfxbuffer 0x%x\n", gfxbuffer);
	clearScreen();
	

#ifdef MIOS_BACKUP
	/* When linked into MIOS, check first if homebrew should be started. */
	check_for_homebrew();
#endif

	if (dvd_read_id() != 0)
	{
		backuploading = true;
		//printf("\n   Loading backup disc...\n");
	} else
	{
		backuploading = false;
		//printf("\n   Loading retail disc...\n");
	}

	if (strcmp((char *)0x807FFF00, "GAMECUBELOADER01") == 0)
	{
		start_from_NeoGamma();
	} 
	
	/* Start game directly. */
	start_backup_loader();

	clearScreen();

	/* Start MIOS without backup support. */
	call_mios();

	SYS_ResetSystem(SYS_RESTART, 0, 0);
	return 0;
}
